@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2008 Erwin Waterlander
+Copyright (C) 2000-2010 Erwin Waterlander
 
 Ideas and source code of NCD (Ninux Czo Directory) have been
 used in the WCD graphical interface.
@@ -61,15 +61,38 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "colors.h"             /* add colors for the tree on MS platform */
 #endif
 
+/*
+ * The following codes are below 20 and don't interfere with 8 bit normal
+ * characters.  They will also not interfere with UTF-8 encoded characters,
+ * because All UCS characters >U+007F are encoded as a sequence of several
+ * bytes, each of which has the most significant bit set. No ASCII byte
+ * (0x00-0x7F) can appear as part of any other UTF-8 encode character.
+ */
+
+# define WCD_ACS_HL 1          /* line art codes */
+# define WCD_ACS_VL 2
+# define WCD_ACS_LT 3
+# define WCD_ACS_LLC 4
+# define WCD_ACS_TT 5
+
 # define WCD_SEL_ON 6           /* selection on/off codes */
 # define WCD_SEL_OFF 7
 
-#define WCD_ONESUBDIR  "---"
-#define WCD_SPLITDIR   "-+-"
-#define WCD_SUBDIR    "  |-"
-#define WCD_MOREDIR   "  |  "
-#define WCD_ENDDIR    "  `-"
-#define WCD_OVERDIR   "     "
+#ifdef ASCII_TREE
+static const char WCD_ONESUBDIR[] = "---" ;
+static const char WCD_SPLITDIR[] =  "-+-" ;
+static const char WCD_SUBDIR[] =   "  |-" ;
+static const char WCD_MOREDIR[] =  "  |  " ;
+static const char WCD_ENDDIR[] =   "  `-" ;
+static const char WCD_OVERDIR[] =  "     " ;
+#else
+static const char WCD_ONESUBDIR[] = { WCD_ACS_HL, WCD_ACS_HL, WCD_ACS_HL, 0} ;
+static const char WCD_SPLITDIR[] =  { WCD_ACS_HL, WCD_ACS_TT, WCD_ACS_HL, 0} ;
+static const char WCD_SUBDIR[] =    { 32, 32, WCD_ACS_LT, WCD_ACS_HL, 0} ;
+static const char WCD_MOREDIR[] =   { 32, 32, WCD_ACS_VL, 32, 32, 0} ;
+static const char WCD_ENDDIR[] =    { 32, 32, WCD_ACS_LLC, WCD_ACS_HL, 0} ;
+static const char WCD_OVERDIR[] =  "     " ;
+#endif
 
 #define WCD_GRAPH_MAX_LINE_LENGTH  DD_MAXPATH * 2
 
@@ -333,6 +356,7 @@ void dumpTree(dirnode d)
    int index, size;
    dirnode n;
    int y;
+   char *l;
 
 
    index = 0;
@@ -350,7 +374,38 @@ void dumpTree(dirnode d)
    else
    {
       y = dirnodeGetY(d);
-      printf("%s\n",getTreeLine(d,y,&y,NULL,false));
+      l = getTreeLine(d,y,&y,NULL,false);
+      while ( *l != '\0')
+         {
+            switch(*l)
+            {
+               case WCD_ACS_HL:
+                putc('-', stdout);
+                  break;
+               case WCD_ACS_VL:
+                putc('|', stdout);
+                  break;
+               case WCD_ACS_LT:
+                putc('|', stdout);
+                  break;
+               case WCD_ACS_LLC:
+                putc('`', stdout);
+                  break;
+               case WCD_ACS_TT:
+                putc('+', stdout);
+                  break;
+               case WCD_SEL_ON:
+                putc('<', stdout);
+                  break;
+               case WCD_SEL_OFF:
+                putc('>', stdout);
+                  break;
+               default:
+                putc(*l, stdout);
+            }
+            l++;
+         }
+      printf("\n");
    }
 
 
@@ -1196,16 +1251,46 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
          {
             switch(s[j])
             {
+               case WCD_ACS_HL:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '-');
+                else
+                  waddch(win,ACS_HLINE);
+                  break;
+               case WCD_ACS_VL:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '|');
+                else
+                  waddch (win,ACS_VLINE);
+                  break;
+               case WCD_ACS_LT:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '|');
+                else
+                  waddch (win,ACS_LTEE);
+                  break;
+               case WCD_ACS_LLC:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '`');
+                else
+                  waddch (win,ACS_LLCORNER);
+                  break;
+               case WCD_ACS_TT:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '+');
+                else
+                  waddch (win,ACS_TTEE);
+                  break;
                case WCD_SEL_ON:
                 wattron(win,A_REVERSE);
-                waddch(win,'[');
+                waddch(win,'['); /* square brackets indicate there was an error. */
                   break;
                case WCD_SEL_OFF:
                 waddch(win,']');
                 wattroff(win,A_REVERSE);
                   break;
                default:
-                  waddch(win,s[j]);
+                waddch(win,s[j]);
             }
          }
       }
@@ -1217,9 +1302,14 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
          {
             switch(wstr[j])
             {
+               case WCD_ACS_HL:
+               case WCD_ACS_VL:
+               case WCD_ACS_LT:
+               case WCD_ACS_LLC:
+               case WCD_ACS_TT:
                case WCD_SEL_ON:
-                  break;
                case WCD_SEL_OFF:
+                  c = c + 1;
                   break;
                default:
                   c = c + wcwidth(wstr[j]);
@@ -1227,23 +1317,26 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
             j++; /* j advances over combining characters */
          }
          /* xoffset is horizontal offset measured in nr. of columns.
-	  * This may be at the middle of a double width character. */
-	 if (( c > xoffset ) && ( wcwidth(wstr[j-1]) == 2))
-	 {
-	    /* Last character skipped was a double width character.
-	     * Insert a space. */
-	    j--;
-	    wstr[j] = ' ';
-	 }
+          * This may be at the middle of a double width character. */
+         if (( c > xoffset ) && ( wcwidth(wstr[j-1]) == 2))
+         {
+            /* Last character skipped was a double width character.
+             * Insert a space. */
+            j--;
+            wstr[j] = ' ';
+         }
          while ((j<len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
            j++;
          width = 0;
          if (j<len)
             switch(wstr[j])
             {
+               case WCD_ACS_HL:
+               case WCD_ACS_VL:
+               case WCD_ACS_LT:
+               case WCD_ACS_LLC:
+               case WCD_ACS_TT:
                case WCD_SEL_ON:
-                width = 1;
-                  break;
                case WCD_SEL_OFF:
                 width = 1;
                   break;
@@ -1254,6 +1347,36 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
          {
             switch(wstr[j])
             {
+               case WCD_ACS_HL:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '-');
+                else
+                  waddch(win,ACS_HLINE);
+                  break;
+               case WCD_ACS_VL:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '|');
+                else
+                  waddch (win,ACS_VLINE);
+                  break;
+               case WCD_ACS_LT:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '|');
+                else
+                  waddch (win,ACS_LTEE);
+                  break;
+               case WCD_ACS_LLC:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '`');
+                else
+                  waddch (win,ACS_LLCORNER);
+                  break;
+               case WCD_ACS_TT:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '+');
+                else
+                  waddch (win,ACS_TTEE);
+                  break;
                case WCD_SEL_ON:
                 wattron(win,A_REVERSE);
                 waddch(win,'<');
@@ -1263,14 +1386,17 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
                 wattroff(win,A_REVERSE);
                   break;
                default:
-                  waddnwstr(win,wstr+j,1);
+                waddnwstr(win,wstr+j,1);
             }
             j++;
             switch(wstr[j])
             {
+               case WCD_ACS_HL:
+               case WCD_ACS_VL:
+               case WCD_ACS_LT:
+               case WCD_ACS_LLC:
+               case WCD_ACS_TT:
                case WCD_SEL_ON:
-                width = width + 1;
-                  break;
                case WCD_SEL_OFF:
                 width = width + 1;
                   break;
@@ -1284,6 +1410,36 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
       {
          switch(s[j])
          {
+            case WCD_ACS_HL:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '-');
+                else
+                  waddch(win,ACS_HLINE);
+               break;
+            case WCD_ACS_VL:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '|');
+                else
+                  waddch (win,ACS_VLINE);
+               break;
+            case WCD_ACS_LT:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '|');
+                else
+                  waddch (win,ACS_LTEE);
+               break;
+            case WCD_ACS_LLC:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '`');
+                else
+                  waddch (win,ACS_LLCORNER);
+               break;
+            case WCD_ACS_TT:
+                if (wcd_cwin.graphics_mode & WCD_GRAPH_ASCII)
+                  waddch(win, '+');
+                else
+                  waddch (win,ACS_TTEE);
+               break;
             case WCD_SEL_ON:
                   wattron(win,A_REVERSE);
                   waddch(win,'<');
@@ -1293,7 +1449,7 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
                   wattroff(win,A_REVERSE);
                break;
             default:
-               waddch(win,s[j]);
+                  waddch(win,s[j]);
          }
       }
 #endif
