@@ -1549,78 +1549,63 @@ void print_help()
 {
         printf(_("\
 wcd  %s  (%s) - Wherever Change Directory\n\
-Usage: wcd [-a[a]] [-A <path>] [-b] [-c] [-d <drive>] [-e[e]] [-E <path>]\n\
-       [-f <treefile>] [-g[a|d]] [-G <path>] [-h] [-i] [-j] [-k] [-l <alias>]\n\
-       [-[m|M] <dir>] [-n <path>] [-N] [-o[d]] [-q] [-[r|rmtree] <dir>] [-s]\n\
-       [-S <path>] [-t] [-u <user>] [-v] [-V] [-w] [-x <path>] [-xf <file>]\n\
-       [-z #] [-[#]] [+[#]] [=] [drive:][dir]\n\
+Usage: wcd [options] [directory]\n\
 \n\
-  [dir]   (partial) name of directory to change to.\n\
-          Wildcards *, ? and [SET] are supported!\n\n"),VERSION,VERSION_DATE);
+directory:  Name of directory to change to.\n\
+            Wildcards *, ? and [SET] are supported.\n\n"),VERSION,VERSION_DATE);
 
   printf(_("options:\n\
   -a             Add current path to treefile\n\
   -aa            Add current and all parent paths to treefile\n\
-  -A             Add tree from <path>\n\
+  -A PATH        Add tree from PATH\n\
   -b             Ban current path\n\
   -c             direct CD mode\n\
-  -d             set <drive> for stack & go files (DOS)\n\
+  -d DRIVE       set DRIVE for stack & go files (DOS)\n\
   -e             add current path to Extra treefile\n\
   -ee            add current and all parent paths to Extra treefile\n\
-  -E             add tree from <path> to Extra treefile\n\
-  -f             use extra treeFile\n\
-  +f             add extra treeFile\n\
+  -E PATH        add tree from PATH to Extra treefile\n\
+  -f FILE        use extra treeFile\n\
+  +f FILE        add extra treeFile\n\
   -g             Graphics\n\
   -ga            Graphics, alternative navigation\n\
   -gc            Graphics, centered\n\
   -gd            Graphics, dump tree\n\
-  -G             set path Go-script\n\
+  -G PATH        set PATH Go-script\n\
   -GN            No Go-script\n\
   -h, --help     show this Help\n\
   -i             Ignore case\n\
   +i             Regard case\n\
   -j             Just go mode\n\
-  -K             Colors\n\
   -k             Keep paths\n\
-  -l             aLias current directory\n\
+  -K             Colors\n\
+  -l ALIAS       aLias current directory\n\
   -L             print software License\n\
-  -m             Make <dir>, add to treefile\n\
-  -M             Make <dir>, add to extra treefile\n\
-  -n             use relative treefile in <path>\n\
-  +n             add relative treefile in <path>\n\
+  -m DIR         Make DIR, add to treefile\n\
+  -M DIR         Make DIR, add to extra treefile\n\
+  -n PATH        use relative treefile in PATH\n\
+  +n PATH        add relative treefile in PATH\n\
   -N             use Numbers\n\
   -o             use stdOut\n\
   -od            dump matches\n\
   -q             Quieter operation\n\
-  -r             Remove <dir>\n\
-  -rmtree        Remove <dir> recursive\n\
+  -r DIR         Remove DIR\n\
+  -rmtree DIR    Remove DIR recursive\n\
   -s             Scan disk from $HOME\n\
-  -S             Scan disk from <path>\n\
-  +S             Scan disk from <path>, create relative treefile\n\
-  -u             use <user> treefile\n\
-  +u             add <user> treefile\n\
+  -S PATH        Scan disk from PATH\n\
+  +S PATH        Scan disk from PATH, create relative treefile\n\
+  -t             Don't strip /tmp_mnt from paths\n\
+  -T             Draw tree with ASCII characters\n\
+  -u USER        use USER's treefile\n\
+  +u USER        add USER's treefile\n\
   -v, --verbose  Verbose operation\n\
   -V, --version  print Version info\n\
   -w             Wild matching only\n\
-  -x             eXclude <path> during disk scan\n\
-  -xf            eXclude paths from <file>\n\
-  -z             set max stack siZe\n\
-  -              Push dir (# times)\n\
-  +              Pop dir (# times)\n\
+  -x PATH        eXclude PATH during disk scan\n\
+  -xf FILE       eXclude paths from FILE\n\
+  -z NUMBER      set max stack siZe\n\
+  -[NUMBER]      Push dir (NUMBER times)\n\
+  +[NUMBER]      Pop dir (NUMBER times)\n\
   =              Show stack\n"));
-#ifdef WCD_WINPWRSH
-   printf(_("This version is for Windows PowerShell.\n"));
-#endif
-#ifdef WCD_WINZSH
-   printf(_("This version is for MSYS and WinZsh.\n"));
-#endif
-#ifdef __MSYS__
-   printf(_("This version is for native MSYS.\n"));
-#endif
-#ifdef WCD_DOSBASH
-   printf(_("This version is for DJGPP DOS bash.\n"));
-#endif
-
 }
 
 #ifdef ENABLE_NLS
@@ -1637,7 +1622,7 @@ void print_version()
 # elif defined(WCD_WINPWRSH)
    printf(_("This version is for Windows PowerShell.\n"));
 # else
-   printf(_("Windows console version.\n"));
+   printf(_("This version is for Windows Command Prompt (cmd.exe).\n"));
 # endif
 # ifdef __WIN64__
    printf(_("Win64 version.\n"));
@@ -1856,7 +1841,7 @@ void writeGoFile(char *go_file, int *changedrive, char *drive, char *best_match,
 #    endif
    fprintf(outfile,"set-location %s", best_match);
 #  else
-   /* Windows console, os/2 */
+   /* Windows Command Prompt, os/2 */
    fprintf(outfile, "%s", "@echo off\n");
    if (*changedrive)
       fprintf(outfile,"%s\n",drive);
@@ -2362,6 +2347,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n"
                 strip_mount_string = 0;
             break;
 #endif
+         case 'T':
+               graphics |= WCD_GRAPH_ASCII ;
+            break;
 #ifdef MSDOS
          case 'd':
             break;
@@ -3019,7 +3007,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n"
          rootNode = createRootNode();
          buildTreeFromNameset(perfect_list, rootNode);
          setXYTree(rootNode);
-         ptr = selectANode(rootNode,&use_HOME,ignore_case,graphics);
+         if (graphics & WCD_GRAPH_DUMP)
+         {
+            dumpTree(rootNode);
+            ptr = NULL;
+         }
+	 else
+            ptr = selectANode(rootNode,&use_HOME,ignore_case,graphics);
          if (ptr != NULL)
             strcpy(best_match,ptr);
          else
@@ -3064,7 +3058,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n"
          rootNode = createRootNode();
          buildTreeFromNameset(wild_list, rootNode);
          setXYTree(rootNode);
-         ptr = selectANode(rootNode,&use_HOME,ignore_case,graphics);
+         if (graphics & WCD_GRAPH_DUMP)
+         {
+            dumpTree(rootNode);
+            ptr = NULL;
+         }
+	 else
+            ptr = selectANode(rootNode,&use_HOME,ignore_case,graphics);
          if (ptr != NULL)
             strcpy(best_match,ptr);
          else
